from setuptools import setup
if __name__ == '__main__':
    setup(
        #packages = ['numtoname'],

        # Stupid stuff I might need for src structure one day
        # py_modules = ['numtoname.functions', 'numtoname.helpers'],
        # package_dir = {'': 'numtoname'},
    )
